@@ -1,13 +1,15 @@
 #' Play audio
 #'
-#' Plays an audio file or a numeric vector. This is a simple wrapper for the
-#' functionality provided by \code{\link[tuneR]{play}}. Recommended players on
-#' Linux: "play" from the "vox" library (default), "aplay".
-#' @param sound a vector of numbers on any scale or a path to a .wav file
+#' Plays an audio file (wav or mp3) or a numeric vector. This is a simple
+#' wrapper for the functionality provided by \code{\link[tuneR]{play}}.
+#' Recommended players on Linux: "play" from the "vox" library (default),
+#' "aplay".
+#' @param sound numeric vector or path to wav/mp3 file
 #' @param samplingRate sampling rate (only needed if sound is a vector)
 #' @param player the name of player to use, eg "aplay", "play", "vlc", etc. In
 #'   case of errors, try setting another default player for
 #'   \code{\link[tuneR]{play}}
+#' @param from,to play a selected time range (s)
 #' @param ... additional parameters passed to \code{\link[tuneR]{play}}
 #' @export
 #' @examples
@@ -18,6 +20,7 @@
 #' f0_Hz = 440
 #' sound = sin(2 * pi * f0_Hz * (1:16000) / 16000)
 #' # playme(sound, 16000)
+#' # playme(sound, 16000, from = .1, to = .5)  # play from 100 to 500 ms
 #'
 #' # In case of errors, look into tuneR::play(). For ex., you might need to
 #' # specify which player to use:
@@ -26,10 +29,21 @@
 #' # To avoid doing it all the time, set the default player:
 #' tuneR::setWavPlayer('aplay')
 #' # playme(sound, 16000)  # should work without specifying the player
-playme = function(sound, samplingRate = 16000, player = NULL) {
+playme = function(sound,
+                  samplingRate = 16000,
+                  player = NULL,
+                  from = NULL,
+                  to = NULL) {
   # input: a vector of numbers on any scale or a path to a .wav file
   if (class(sound) == 'character') {
-    soundWave = tuneR::readWave(sound)
+    extension = substr(sound, nchar(sound) - 2, nchar(sound))
+    if (extension == 'wav' | extension == 'WAV') {
+      soundWave = tuneR::readWave(sound)
+    } else if (extension == 'mp3' | extension == 'MP3') {
+      soundWave = tuneR::readMP3(sound)
+    } else {
+      stop('Input not recognized: must be a numeric vector or wav/mp3 file')
+    }
   } else if (class(sound) == 'numeric' | class(sound) == 'integer') {
     soundWave = tuneR::Wave(
       left = sound,
@@ -40,6 +54,17 @@ playme = function(sound, samplingRate = 16000, player = NULL) {
     soundWave = tuneR::normalize(soundWave, unit = '32') # / 2
   }
 
+  # to play a selected time range
+  if (!is.null(from) | !is.null(to)) {
+    if (is.null(from)) from = 0
+    if (is.null(to)) to = length(soundWave@left) / soundWave@samp.rate
+    soundWave = tuneR::extractWave(object = soundWave,
+                                   from = from,
+                                   to = to,
+                                   interact = FALSE,
+                                   xunit = 'time')
+  }
+
   if (!is.null(player)) {
     p = tuneR::play(soundWave, player = player)
   } else {
