@@ -8,12 +8,13 @@
 #' reasonable guess and computes the more-or-less optimal pitch contour (not
 #' quite the very optimal - too computationally expensive).
 #' @param pitchCands a matrix of multiple pitch candidates per fft frame. Each
-#'   column is one fft frame, each row is one candidate.
+#'   column is one fft frame, each row is one candidate (the last row is always
+#'   "manual")
 #' @param pitchCert a matrix of the same dimensionality as pitchCands specifying
 #'   our certainty in pitch candidates
 #' @inheritParams analyze
-#' @param interpolWin when interpolating pitch candidates, the median is
-#'   calculated over \code{plus-minus interpolWin}
+#' @param interpolWin_bin when interpolating pitch candidates, the median is
+#'   calculated over \code{plus-minus interpolWin_bin}
 #' @param interpolTol when interpolating pitch candidates, the criterion
 #'   for needing to interpolate is the absence of pitch candidates with values
 #'   within \code{1 plus-minus interpolTol} of the median of pitch center of
@@ -22,67 +23,98 @@
 #'   interpolation window.
 #' @param interpolCert when interpolating pitch candidates, all generated pitch
 #'   candidates are assigned a certainty equal to \code{interpolCert}
+#' @param manualCert the certainty in manually added pitch candidates
 #' @return Returns a numeric vector of pitch values representing the best found
 #'   path through pitch candidates.
 #' @keywords internal
 pathfinder = function(pitchCands,
                       pitchCert,
+                      pitchSource,
+                      manual = NULL,
                       certWeight = 0.5,
                       pathfinding = c('none', 'fast', 'slow')[2],
                       annealPars = list(maxit = 5000, temp = 1000),
-                      interpolWin = 3,
+                      interpolWin_bin = 3,
                       interpolTol = 0.05,
                       interpolCert = 0.3,
+                      manualCert = 1,
                       snakeStep = 0.05,
                       snakePlot = FALSE) {
+  nr = nrow(pitchCands)
+  nc = ncol(pitchCands)
+
+  # Add "inviolable" manual pitch values
+  if (length(manual$frame) > 0) {
+    freqMan = certMan = rep(NA, nc)
+    freqMan[manual$frame] = manual$freq
+    certMan[manual$frame] = manualCert
+    pitchCands = rbind(pitchCands, freqMan)
+    pitchCert = rbind(pitchCert, certMan)
+    pitchSource = rbind(pitchSource, rep('manual', nc))
+    nr = nr + 1
+  }
+
   # take log to approximate human perception of pitch differences
   pitchCands[!is.na(pitchCands)] = log2(pitchCands[!is.na(pitchCands)])
+  if (!is.null(manual)) manual$freq = log2(manual$freq)
 
   # get the "center of gravity" of pitch candidates in each frame (mean of all
   # pitch candidates weighted by their respective certainties)
-  pitchCenterGravity = apply(as.matrix(1:ncol(pitchCands), nrow = 1), 1, function(x) {
-    mean(
-      pitchCands[, x],
-      weights = pitchCert[, x] / sum(pitchCert[, x]),
-      na.rm = T
-    )
-  })
+  pitchCenterGravity = rep(NA, ncol(pitchCands))
+  for (i in 1:ncol(pitchCands)) {
+    idxDeadCert = which(pitchSource[, i] == 'manual' & pitchCert[, i] == manualCert)
+    if (length(idxDeadCert) == 1) {
+      # special case: pitchCenterGravity has to pass through manual candidates
+      pitchCenterGravity[i] = pitchCands[idxDeadCert, i]
+    } else {
+      pitchCenterGravity[i] = weighted.mean(x = pitchCands[, i],
+                                            w = pitchCert[, i],
+                                            na.rm = TRUE)
+    }
+  }
 
   ## INTERPOLATION
   # if a frame has no pitch candidate at all (NA) or no candidate
   # between the most likely candidates for the adjacent frames, add such a
   # candidate with ~low certainty
-  if (is.numeric(interpolWin)) {
-    if (interpolWin > 0) {
-      intplt = interpolate(pitchCands = pitchCands,
-                           pitchCert = pitchCert,
-                           pitchCenterGravity = pitchCenterGravity,
-                           interpolWin = interpolWin,
-                           interpolTol = interpolTol,
-                           interpolCert = interpolCert)
-      pitchCands = intplt$pitchCands
-      pitchCert = intplt$pitchCert
-      pitchCenterGravity = intplt$pitchCenterGravity
+  if (interpolWin_bin > 0) {
+    # order pitch candidates and certainties in each frame pushing NAs down so
+    # as to simplify the matrix (the position of manual candidates is not
+    # important since they are saved in 'manual')
+    if (nrow(pitchCands) > 1) {
+      o = apply(as.matrix(1:ncol(pitchCands), nrow = 1), 1, function(x) {
+        order(pitchCands[, x])
+      })
+      pitchCands = apply(matrix(1:ncol(pitchCands)), 1, function(x) {
+        pitchCands[o[, x], x]
+      })
+      pitchCert = apply(matrix(1:ncol(pitchCert)), 1, function(x) {
+        pitchCert[o[, x], x]
+      })
+      pitchSource = apply(matrix(1:ncol(pitchSource)), 1, function(x) {
+        pitchSource[o[, x], x]
+      })
     }
-  }
-
-  # order pitch candidates and certainties in each frame from lowest to highest
-  # pitch (helpful for further processing)
-  if (nrow(pitchCands) > 1) {
-    o = apply(as.matrix(1:ncol(pitchCands), nrow = 1), 1, function(x) {
-      order(pitchCands[, x])
-    })
-    pitchCands = apply(matrix(1:ncol(pitchCands)), 1, function(x) {
-      pitchCands[o[, x], x]
-    })
-    pitchCert = apply(matrix(1:ncol(pitchCert)), 1, function(x) {
-      pitchCert[o[, x], x]
-    })
+    intplt = interpolate(pitchCands = pitchCands,
+                         pitchCert = pitchCert,
+                         pitchSource = pitchSource,
+                         pitchCenterGravity = pitchCenterGravity,
+                         interpolWin_bin = interpolWin_bin,
+                         interpolTol = interpolTol,
+                         interpolCert = interpolCert)
+    pitchCands = intplt$pitchCands
+    pitchCert = intplt$pitchCert
+    pitchSource = intplt$pitchSource
+    pitchCenterGravity = intplt$pitchCenterGravity
   }
 
   # remove rows with all NA's
-  pitchCands = pitchCands[rowSums(!is.na(pitchCands)) != 0, , drop = FALSE]
-  pitchCert = pitchCert[rowSums(!is.na(pitchCert)) != 0, , drop = FALSE]
+  keep_rows = which(rowSums(!is.na(pitchCands)) > 0)
+  if (length(keep_rows) > 0) {
+    pitchCands = pitchCands[keep_rows, , drop = FALSE]
+    pitchCert = pitchCert[keep_rows, , drop = FALSE]
+    pitchSource = pitchSource[keep_rows, , drop = FALSE]
+  }
 
   # special case: only a single pitch candidate for all frames in a syllable
   # (no paths to chose among)
@@ -92,11 +124,13 @@ pathfinder = function(pitchCands,
 
   ## PATH-FINDING
   # find the best path through frame-by-frame pitch candidates
-  if (ncol(pitchCands) < 2) pathfinding = 'none'
+  if (nc < 2) pathfinding = 'none'
   if (pathfinding == 'fast') {
     bestPath = pathfinding_fast(
       pitchCands = pitchCands,
       pitchCert = pitchCert,
+      pitchSource = pitchSource,
+      manual = manual,
       pitchCenterGravity = pitchCenterGravity,
       certWeight = certWeight
     )
@@ -106,30 +140,32 @@ pathfinder = function(pitchCands,
       pitchCert = pitchCert,
       certWeight = certWeight,
       pitchCenterGravity = pitchCenterGravity,
+      manual = manual,
       annealPars = annealPars
     )
-  } else {
+  } else {  # if (pathfinding == 'none')
     bestPath = apply(matrix(1:ncol(pitchCands)), 1, function(x) {
       idx = which.min(abs(pitchCands[, x] - pitchCenterGravity[x]))
       pitchCands[idx, x]
-    }) # or bestPath = pitchCenterGravity
+    })
+    if (length(manual$frame) > 0) {
+      bestPath[manual$frame] = manual$freq
+    }
   }
 
   ## SNAKE
   # apply the snake algorithm to minimize the elastic forces acting on this
   # pitch contour without deviating too far from high-certainty anchors
-  if (is.numeric(snakeStep)) {
-    if (snakeStep > 0) {
-      bestPath = snake(
-        pitch = bestPath,
-        pitchCands = pitchCands,
-        pitchCert = pitchCert,
-        certWeight = certWeight,
-        pitchCenterGravity = pitchCenterGravity,
-        snakeStep = snakeStep,
-        snakePlot = snakePlot
-      )
-    }
+  if (snakeStep > 0) {
+    bestPath = snake(
+      pitch = bestPath,
+      pitchCands = pitchCands,
+      pitchCert = pitchCert,
+      certWeight = certWeight,
+      pitchCenterGravity = pitchCenterGravity,
+      snakeStep = snakeStep,
+      snakePlot = snakePlot
+    )
   }
 
   return(2 ^ bestPath)
@@ -148,19 +184,34 @@ pathfinder = function(pitchCands,
 #' @param pitchCenterGravity numeric vector giving the mean of all pitch
 #'   candidates per fft frame weighted by our certainty in each of these
 #'   candidates
+#' @param manualUnvoiced a vector with indices of frames that are manually
+#'   specified as unvoiced and should not be interpolated
 #' @return Returns a modified pitchCands matrix.
 #' @keywords internal
 interpolate = function(pitchCands,
                        pitchCert,
+                       pitchSource,
                        pitchCenterGravity,
-                       interpolWin = 3,
+                       interpolWin_bin = 3,
                        interpolTol = 0.3,
                        interpolCert = 0.3) {
-  for (f in 1:ncol(pitchCands)) {
-    left = max(1, f - interpolWin)
-    right = min(ncol(pitchCands), f + interpolWin)
+  # ... add an empty row for new, interpolated pitch candidates
+  nc = ncol(pitchCands)
+  pitchCands = rbind(rep(NA, nc), pitchCands)
+  pitchCert = rbind(rep(NA, nc), pitchCert)
+  pitchSource = rbind(rep('intpl', nc), pitchSource)
+  for (f in 1:nc) {
+    left = max(1, f - interpolWin_bin)
+    right = min(ncol(pitchCands), f + interpolWin_bin)
     # median over interpolation window (by default plus-minus 2 points)
-    med = median(pitchCenterGravity[left:right], na.rm = TRUE)
+    idx_man = which(pitchSource[, left:right] == 'manual' &
+                      !is.na(pitchCert[, left:right]))
+    if (length(idx_man) > 0) {
+      # special case: if any manual cand-s over interpolWin_bin, only consider them
+      med = median(pitchCands[, left:right] [idx_man], na.rm = TRUE)
+    } else {
+      med = median(pitchCenterGravity[left:right], na.rm = TRUE)
+    }
     sum_pitchCands = sum(
       pitchCands[, f] > (1 - interpolTol) * med &
         pitchCands[, f] < (1 + interpolTol) * med,
@@ -169,21 +220,23 @@ interpolate = function(pitchCands,
     if (sum_pitchCands == 0 & !is.na(med)) {
       # if there are no pitch candidates in the frequency range
       # expected based on pitch candidates in the adjacent frames...
-      # ... add an empty row for a new, interpolated pitch candidate
-      pitchCands = rbind(pitchCands, rep(NA, ncol(pitchCands)))
-      pitchCert = rbind(pitchCert, rep(NA, ncol(pitchCert)))
       # use median of adjacent frames for the new pitch cand
-      pitchCands[nrow(pitchCands), f] = med
+      pitchCands[1, f] = med
       # certainty assigned to interpolated frames
-      pitchCert[nrow(pitchCert), f] = interpolCert
+      pitchCert[1, f] = interpolCert
       # update pitchCenterGravity for the interpolated frame
-      pitchCenterGravity[f] = mean(pitchCands[, f],
-                                   weights = pitchCert[, f] / sum(pitchCert[, f]),
-                                   na.rm = TRUE)
+      if (length(idx_man) > 0) {
+        pitchCenterGravity[f] = med
+      } else {
+        pitchCenterGravity[f] = weighted.mean(x = pitchCands[, f],
+                                              w = pitchCert[, f],
+                                              na.rm = TRUE)
+      }
     }
   }
   return(list(pitchCands = pitchCands,
               pitchCert = pitchCert,
+              pitchSource = pitchSource,
               pitchCenterGravity = pitchCenterGravity))
 }
 
@@ -203,24 +256,34 @@ interpolate = function(pitchCands,
 #' @param pitchCenterGravity numeric vector giving the mean of all pitch
 #'   candidates per fft frame weighted by our certainty in each of these
 #'   candidates
+#' @param manual dataframe giving manual pitch candidates, which the path
+#'   MUST go through
 #' @keywords internal
-pathfinding_fast = function(pitchCands = pitchCands,
-                            pitchCert = pitchCert,
-                            pitchCenterGravity = pitchCenterGravity,
-                            certWeight = certWeight) {
-  # start at the beginning of the snake: find the most plausible starting pitch
-  # by taking median over the first few frames, weighted by certainty
-  p = median(pitchCenterGravity[1:min(5, length(pitchCenterGravity))],
-             na.rm = TRUE)
-  c = pitchCert[, 1] / abs(pitchCands[, 1] - p) # b/c there may be NA's,
-  # and they can't be excluded directly in which.max in the next line
-  point_current = pitchCands[which.max(c), 1]
+pathfinding_fast = function(pitchCands,
+                            pitchCert,
+                            pitchSource,
+                            manual,
+                            pitchCenterGravity,
+                            certWeight) {
+  nc = ncol(pitchCands)
+  nr = nrow(pitchCands)
+
+  # find the most plausible starting pitch by taking median over the first few
+  # frames, weighted by certainty
+  if (1 %in% manual$frame) {
+    point_current = manual$freq[manual$frame == 1]
+  } else {
+    p = median(pitchCenterGravity[1:min(5, nc)], na.rm = TRUE)
+    c = pitchCert[, 1] / abs(pitchCands[, 1] - p)
+    point_current = pitchCands[which.max(c), 1]
+  }
   path = point_current
   costPathForward = 0
 
-  for (i in 2:ncol(pitchCands)) {
-    cands = na.omit(pitchCands[, i])
-    if (length(cands) > 0) { # in case of an NA in the contour
+  # run forwards
+  for (i in 2:nc) {
+    cands = pitchCands[, i]
+    if (any(!is.na(cands))) {
       cost_cert = abs(cands - pitchCenterGravity[i])
       # get the cost of transition from the current point to each of the pitch
       # candidates in the next frame
@@ -231,37 +294,62 @@ pathfinding_fast = function(pitchCands = pitchCands,
       # get a weighted average of transition costs associated with the certainty
       # of each estimate vs. the magnitude of pitch jumps
       costs = certWeight * cost_cert + (1 - certWeight) * cost_pitchJump
-      path = c(path, pitchCands[which.min(costs), i])
-      costPathForward = costPathForward + min(costs)
+      if (i %in% manual$frame) {
+        idx = which(pitchSource[, i] == 'manual')
+      } else {
+        idx = which.min(costs)
+      }
+      point_current = pitchCands[idx, i]
+      path = c(path, point_current)
+      costPathForward = costPathForward + costs[idx]
+    } else {
+      path = c(path, NA)
     }
   }
 
   # run backwards
-  pitchCands_rev = pitchCands[, rev(1:ncol(pitchCands)), drop = FALSE]
-  pitchCert_rev = pitchCert[, rev(1:ncol(pitchCert)), drop = FALSE]
+  pitchCands_rev = pitchCands[, rev(1:nc), drop = FALSE]
+  pitchCert_rev = pitchCert[, rev(1:nc), drop = FALSE]
+  pitchSource_rev = pitchSource[, rev(1:nc), drop = FALSE]
   pitchCenterGravity_rev = rev(pitchCenterGravity)
+  manual_rev = manual
+  manual_rev$frame = nc - manual$frame + 1
 
-  p = median (pitchCenterGravity_rev[1:min(5, length(pitchCenterGravity_rev))])
-  c = na.omit (pitchCert_rev[, 1] / abs(pitchCands_rev[, 1] - p)) # b/c there may be NA's,
-  # and they can't be excluded directly in which.max in the next line
-  point_current = pitchCands_rev[which.max(c), 1]
+  if (1 %in% manual_rev$frame) {
+    point_current = manual_rev$freq[manual_rev$frame == 1]
+  } else {
+    p = median(pitchCenterGravity_rev[1:min(5, nc)], na.rm = TRUE)
+    c = pitchCert_rev[, 1] / abs(pitchCands_rev[, 1] - p)
+    point_current = pitchCands_rev[which.max(c), 1]
+  }
   path_rev = point_current
   costPathBackward = 0
 
-  for (i in 2:ncol(pitchCands_rev)) {
-    cands = na.omit(pitchCands_rev[, i])
-    if (length(cands) > 0) { # in case of an NA in the contour
+  for (i in 2:nc) {
+    cands = pitchCands_rev[, i]
+    if (any(!is.na(cands))) {
       cost_cert = abs(cands - pitchCenterGravity_rev[i])
       cost_pitchJump = apply(as.matrix(1:length(cands), nrow = 1), 1, function(x) {
         costJumps(point_current, cands[x])
       })
       if (length(cost_pitchJump) == 0) cost_pitchJump = 0
       costs = certWeight * cost_cert + (1 - certWeight) * cost_pitchJump
-      path_rev = c(path_rev, pitchCands_rev[which.min(costs), i])
-      costPathBackward = costPathBackward + min(costs)
+      if (i %in% manual_rev$frame) {
+        idx = which(pitchSource_rev[, i] == 'manual')
+      } else {
+        idx = which.min(costs)
+      }
+      point_current = pitchCands_rev[idx, i]
+      path_rev = c(path_rev, point_current)
+      costPathBackward = costPathBackward + costs[idx]
+    } else {
+      path_rev = c(path_rev, NA)
     }
   }
-
+  #
+  #   er = try(costPathForward < costPathBackward, silent = TRUE)
+  #   if (class(er) == 'try-error' | is.na(er)) browser()
+  if (length(costPathForward) != 1 | length(costPathBackward) != 1) browser()
   if (costPathForward < costPathBackward) {
     bestPath = path
   } else {
@@ -285,7 +373,7 @@ pathfinding_fast = function(pitchCands = pitchCands,
 #' b = 1 / (1 + 10 * exp(3 - 7 * abs(a)))
 #' plot(a, b, type = 'l')
 costJumps = function(cand1, cand2) {
-  return (1 / (1 + 10 * exp(3 - 7 * abs(cand1 - cand2))))
+  return(1 / (1 + 10 * exp(3 - 7 * abs(cand1 - cand2))))
 }
 
 
@@ -301,11 +389,13 @@ costJumps = function(cand1, cand2) {
 #' @param pitchCenterGravity numeric vector giving the mean of all pitch
 #'   candidates per fft frame weighted by our certainty in each of these
 #'   candidates
+#' @param manual a dataframe of manual pitch candidates from pathfinder()
 #' @keywords internal
 pathfinding_slow = function(pitchCands = pitchCands,
                             pitchCert = pitchCert,
                             certWeight = certWeight,
                             pitchCenterGravity = pitchCenterGravity,
+                            manual = NULL,
                             annealPars = list(maxit = 5000, temp = 1000)) {
   # start with the pitch contour most faithful to center of gravity of pitch
   # candidates for each frame
@@ -322,6 +412,7 @@ pathfinding_slow = function(pitchCands = pitchCands,
     pitchCert = pitchCert,
     certWeight = certWeight,
     pitchCenterGravity = pitchCenterGravity,
+    manual = manual,
     method = 'SANN',
     control = annealPars
   )
@@ -347,15 +438,28 @@ pathfinding_slow = function(pitchCands = pitchCands,
 #' @param pitchCenterGravity numeric vector giving the mean of all pitch
 #'   candidates per fft frame weighted by our certainty in each of these
 #'   candidates
+#' @param manual a dataframe of manual pitch candidates from pathfinder()
 #' @keywords internal
 costPerPath = function(path,
                        pitchCands,
                        pitchCert,
                        certWeight,
-                       pitchCenterGravity) {
+                       pitchCenterGravity,
+                       manual = NULL) {
   # if there is nothing to wiggle, generatePath() returns NA and we want
   # annealing to terminate quickly, so we return very high cost
   if (is.na(path[1])) return(1e10)
+
+  # if the path fails to pass through any of the manual pitch values,
+  # also return immediately with very high cost
+  if (!is.null(manual)) {
+    # the last cand in every column is the manual one, so the path
+    # should always go through the last cand in the manual frames
+    if (any(path[manual$frame] != nrow(pitchCands))) {
+      return(1e10)
+    }
+  }
+
   # get the cost of transition from the current point to each of the pitch
   # candidates in the next frame
   cost_pitchJump = apply(as.matrix(1:(length(path) - 1), nrow = 1), 1, function(x) {
@@ -536,7 +640,7 @@ forcePerPath = function (pitch,
 #' @return Returns a vector of the same length as input path giving its 4th derivative.
 #' @keywords internal
 findGrad = function(path, interpol = 3) {
-  # interpolate 2 values before the first one and two after the last one based
+  # extrapolate 2 values before the first one and two after the last one based
   # on /interpol/ number of points in case the path is shorter than the
   # specified interpol:
   interpol = ifelse(interpol > length(path), length(path), interpol)
@@ -554,7 +658,7 @@ findGrad = function(path, interpol = 3) {
     path = c (minus12[2], minus12[1], path, plus12[1], plus12[2])
   }
 
-  # take the 4th derivative of the path with interpolated values
+  # take the 4th derivative of the path with extrapolated values
   # (so that we get d4f over the entire length of the original path)
   grad = rep(0, length(path))
   for (i in 3:(length(path) - 2)) {  # approximation
@@ -577,19 +681,26 @@ findGrad = function(path, interpol = 3) {
 #'   contours can be fed into this function at once to speed things up)
 #' @param smoothing_ww width of smoothing window (points)
 #' @param smoothingThres tolerated deviance from moving median (semitones)
+#' @param inviolable a vector of indices of the rows rows of df that should not
+#'   be modified (meant for manual pitch values)
 #' @return Returns a dataframe of the same dimensions as df.
 #' @keywords internal
 #' @examples
-#' df = data.frame(a = rnorm(100, mean = 100, sd = 20),
-#'                 b = rnorm(100, mean = 100, sd = 10))
-#' df1 = soundgen:::medianSmoother(df, smoothing_ww = 5, smoothingThres = 1)
+#' df = data.frame(a = rnorm(20, mean = 100, sd = 20),
+#'                 b = rnorm(20, mean = 100, sd = 10))
+#' df1 = soundgen:::medianSmoother(df, smoothing_ww = 5,
+#'       smoothingThres = 1,
+#'       inviolable = c(rep(TRUE, 10), rep(FALSE, 10)))
 #' plot(df[, 2], type='b')
 #' lines(df1[, 2], type='b', col='blue', pch=3)
-medianSmoother = function (df, smoothing_ww, smoothingThres) {
+medianSmoother = function(df,
+                          smoothing_ww,
+                          smoothingThres,
+                          inviolable = NULL) {
   temp = df # to calculate median_over_window for original values
   hw = floor(smoothing_ww / 2) # smooth over plus-minus half the smoothing_ww
   for (i in 1:nrow(df)) {
-    window = c (max(i - hw, 1), min(i + hw, nrow(df))) # smoothing window
+    window = c(max(i - hw, 1), min(i + hw, nrow(df))) # smoothing window
     median_over_window = apply(as.matrix(temp[(window[1]:window[2]), ]), 2, function(x) {
       median(unlist(x), na.rm = TRUE)  # w/o unlist returns NULL for NA vectors (weird...)
       # NB: use either temp or df, for original or smoothed values to be used
@@ -597,8 +708,10 @@ medianSmoother = function (df, smoothing_ww, smoothingThres) {
     })
     # difference from median pitch etc over window, in semitones
     deviance = 12 * log2(as.numeric(df[i, ]) / median_over_window)
-    cond = which(abs(deviance - 1) > smoothingThres)
-    df[i, cond] = median_over_window[cond]
+    if (!i %in% inviolable) {
+      cond = which(abs(deviance - 1) > smoothingThres)
+      df[i, cond] = median_over_window[cond]
+    }
   }
   return (df)
 }
@@ -610,7 +723,7 @@ medianSmoother = function (df, smoothing_ww, smoothingThres) {
 #'
 #' Internal helper function for postprocessing of pitch contours. Merges voiced
 #' segments at least \code{shortestSyl} ms long and separated by less than
-#' \code{shortestPause} ms.
+#' \code{shortestPause} ms. Called by \code{\link{analyze}}
 #' @param pitchCands matrix of possible pitch values per column. One column is
 #'   one fft frame, one row is one pitch candidate
 #' @inheritParams analyze
@@ -620,6 +733,7 @@ medianSmoother = function (df, smoothing_ww, smoothingThres) {
 #'   many pitch candidates are not NA. Defaults to 2: since dom is usually
 #'   defined, in practice this means that we also want at least one other pitch
 #'   candidate (autocor, cep or BaNa)
+#' @param pitchMethods methods of pitch tracking in analyze()
 #' @return Returns a dataframe specifying where each voiced segment starts and
 #'   ends (in fft frames, not ms!)
 #' @keywords internal
@@ -628,25 +742,53 @@ findVoicedSegments = function(pitchCands,
                               shortestPause,
                               step,
                               samplingRate,
-                              minVoicedCands) {
-  putativelyVoiced = apply(pitchCands, 2, function(x) {
-    ifelse(sum(!is.na(x)) >= minVoicedCands, 1, NA)
-  })
+                              minVoicedCands,
+                              pitchMethods,
+                              manualV = NULL,
+                              manualUnv = NULL) {
+  resetMVC = FALSE
+  messageMVC = FALSE
+  if (is.null(minVoicedCands)) {
+    resetMVC = TRUE
+  }
+  if (is.numeric(minVoicedCands)) {
+    if (minVoicedCands < 1 | minVoicedCands > length(pitchMethods)) {
+      resetMVC = TRUE
+      messageMVC = TRUE
+    }
+  }
+  if (resetMVC) {
+    if ('dom' %in% pitchMethods & length(pitchMethods) > 1) {
+      # since dom is usually defined, we want at least one more pitch candidate
+      # (unless dom is the ONLY method that the user wants for pitch tracking)
+      minVoicedCands = 2
+    } else {
+      minVoicedCands = 1
+    }
+    if (messageMVC) {
+      message(paste0('minVoicedCands must be between 1 and length(pitchMethods);',
+                     ' resetting to ', minVoicedCands))
+    }
+  }
+  putativelyVoiced = apply(pitchCands, 2, function(x) sum(!is.na(x)) >= minVoicedCands)
+  if (length(manualV) > 0)   putativelyVoiced[manualV]   = TRUE
+  if (length(manualUnv) > 0) putativelyVoiced[manualUnv] = FALSE
+
   # the smallest number of consecutive non-NA pitch values that constitute a
   # voiced segment; but at least 1
-  noRequired = max(1, ceiling(shortestSyl / step))
+  nRequired = max(1, ceiling(shortestSyl / step))
   # the greatest number of NA values that we tolerate before we say a new voiced
   # syllable begins
-  nToleratedNA = floor(shortestPause / step)
+  toleratedGap = floor(shortestPause / step)
 
   # find and save separately all voiced segments
   segmentStart = numeric()
   segmentEnd = numeric()
   i = 1
-  while (i < (length(putativelyVoiced) - noRequired + 1)) {
+  while (i < (length(putativelyVoiced) - nRequired + 1)) {
     # find beginning
-    while (i < (length(putativelyVoiced) - noRequired + 1)) {
-      if (sum(!is.na(putativelyVoiced[i:(i + noRequired - 1)])) == noRequired) {
+    while (i < (length(putativelyVoiced) - nRequired + 1)) {
+      if (sum(putativelyVoiced[i:(i + nRequired - 1)]) == nRequired) {
         segmentStart = c(segmentStart, i)
         break
       }
@@ -654,8 +796,9 @@ findVoicedSegments = function(pitchCands,
     }
     # find end
     if (length(segmentEnd) < length(segmentStart)) {
-      while (i < (length(putativelyVoiced) - nToleratedNA + 1)) {
-        if (sum(putativelyVoiced[i:(i + nToleratedNA)], na.rm = TRUE) == 0) {
+      while (i < (length(putativelyVoiced) - toleratedGap + 1)) {
+        if (sum(putativelyVoiced[i:(i + toleratedGap)]) == 0 |
+            i %in% manualUnv ) {  # interrupt the syllable if we hit a manuaUnv frame
           segmentEnd = c(segmentEnd, i - 1)
           i = i - 1
           break
@@ -663,12 +806,133 @@ findVoicedSegments = function(pitchCands,
         i = i + 1
       }
       if (length(segmentEnd) < length(segmentStart)) {
-        # if the end is not found, take the last non-NA value
-        segmentEnd = c(segmentEnd, tail(which(!is.na(putativelyVoiced)), 1))
+        # if the end is not found, take the last voiced value
+        segmentEnd = c(segmentEnd, tail(which(putativelyVoiced), 1))
         break
       }
     }
     i = i + 1
   }
-  return (data.frame(segmentStart = segmentStart, segmentEnd = segmentEnd))
+  return(data.frame(segmentStart = segmentStart, segmentEnd = segmentEnd))
+}
+
+
+#' Plot pitch candidates
+#'
+#' Internal soundgen function.
+#'
+#' Plots pitch candidates or adds them to a spectrogram.
+#' @param pitchCands,pitchCert,pitchSource matrices of pitch candidates, their
+#'   certainty, and pitch tracking method used as generated internally by
+#'   analyze(); columns = STFT frames, rows = candidates
+#' @param pitch best guess at pitch contour; length = ncol(pitchCands)
+#' @param candPlot,pitchPlot lists of graphical settings for plotting candidates
+#'   and pitch contour, respectively
+#' @param addToExistingPlot if TRUE, assumes that a spectrogram is already
+#'   plotted; if FALSE, sets up a new plot
+#' @param showLegend if TRUE, shows a legend
+#' @param ... other graphical parameters used for creating a new plot if
+#'   addToExistingPlot = FALSE
+#' @examples
+#' \dontrun{
+#' s = soundgen()
+#' a = analyze(s, 16000, windowLength = 25, step = 25,
+#'             summary = 'extended', plot = FALSE)
+#' spectrogram(s, 16000, windowLength = 25, step = 5)
+#' addPitchCands(pitchCands = a$pitchCands,
+#'               pitchCert = a$pitchCert,
+#'               pitchSource = a$pitchSource,
+#'               pitch = a$result$pitch)
+#' }
+addPitchCands = function(pitchCands,
+                         pitchCert,
+                         pitchSource,
+                         pitch,
+                         candPlot = list(),
+                         pitchPlot = list(
+                           col = rgb(0, 0, 1, .75),
+                           lwd = 3
+                         ),
+                         addToExistingPlot = TRUE,
+                         showLegend = TRUE,
+                         ...) {
+  if (length(pitchCands) < 1) stop()
+  # if plot_spec is FALSE, we first have to set up an empty plot
+  if (addToExistingPlot == FALSE) {
+    arguments = list(...)  # save ... arguments as a list
+    m = max(pitchCands, na.rm = TRUE) / 1000  # for ylim on the empty plot
+    if (is.na(m)) m = 5  # samplingRate / 2 / 1000
+    arguments$ylim = c(0, m)
+    do.call(plot,   # need do.call, otherwise can't pass the modified ylim
+            args = c(list(x = as.numeric(colnames(pitchCands)),
+                          y = rep(0, ncol(pitchCands)),
+                          type = 'n'),
+                     arguments))
+  }
+  # add pitch candidates to the plot
+  pitchMethods = unique(na.omit(as.character(pitchSource)))
+  if (any(!is.na(pitchCands))) {
+    if (is.null(candPlot$levels)) {
+      candPlot$levels = pitchMethods # c('autocor', 'spec', 'dom', 'cep')
+    }
+    if (is.null(candPlot$col)) {
+      candPlot$col[candPlot$levels == 'autocor'] = 'green'
+      candPlot$col[candPlot$levels == 'spec'] = 'red'
+      candPlot$col[candPlot$levels == 'dom'] = 'orange'
+      candPlot$col[candPlot$levels == 'cep'] = 'violet' # c('green', 'red', 'orange', 'violet')
+      candPlot$col[candPlot$levels == 'manual'] = 'blue'
+    }
+    if (is.null(candPlot$pch)) {
+      candPlot$pch[candPlot$levels == 'autocor'] = 16
+      candPlot$pch[candPlot$levels == 'spec'] = 2
+      candPlot$pch[candPlot$levels == 'dom'] = 3
+      candPlot$pch[candPlot$levels == 'cep'] = 7
+      candPlot$pch[candPlot$levels == 'manual'] = 18
+      # candPlot$pch = c(16, 2, 3, 7)
+    }
+    if (is.null(candPlot$cex)) {
+      candPlot$cex = 2
+    }
+    pitchSource_1234 = matrix(match(pitchSource, candPlot$levels),
+                              ncol = ncol(pitchSource))
+    for (r in 1:nrow(pitchCands)) {
+      cex = pitchCert[r, ] * candPlot$cex
+      cex[!is.numeric(cex)] = 2  # for manual, certainty could be Inf
+      points(
+        x = as.numeric(colnames(pitchCands)),
+        y = pitchCands[r, ] / 1000,
+        col = candPlot$col[pitchSource_1234[r, ]],
+        pch = candPlot$pch[pitchSource_1234[r, ]],
+        cex = pitchCert[r, ] * candPlot$cex
+      )
+    }
+    # add the final pitch contour to the plot
+    if (any(is.numeric(pitch))) {
+      if (is.null(pitchPlot$col)) {
+        pitchPlot$col = rgb(0, 0, 1, .75)
+      }
+      if (is.null(pitchPlot$lwd)) {
+        pitchPlot$lwd = 3
+      }
+      do.call('lines', c(list(
+        x = as.numeric(colnames(pitchCands)),
+        y = pitch / 1000
+      ),
+      pitchPlot)
+      )
+    }
+    # add a legend
+    if (showLegend) {
+      candPlot = as.data.frame(candPlot)
+      candPlot = candPlot[candPlot$levels %in% c(pitchMethods, 'combined'), ]
+      legend("topright",
+             legend = c(as.character(candPlot$levels), 'combined'),
+             pch = c(candPlot$pch, NA),
+             lty = c(rep(NA, length(pitchMethods)),
+                     ifelse(!is.null(pitchPlot$lty), pitchPlot$lty, 1)),
+             lwd = c(rep(NA, length(pitchMethods)), pitchPlot$lwd),
+             col = c(as.character(candPlot$col), pitchPlot$col),
+             bg = "white")
+    }
+  }
 }
